@@ -54,7 +54,7 @@ static data_unset *configparser_get_variable(config_t *ctx, const buffer *key) {
   return NULL;
 }
 
-/* op1 is to be eat/return by this function, op1->key is not cared
+/* op1 is to be eat/return by this function if success, op1->key is not cared
    op2 is left untouch, unreferenced
  */
 data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
@@ -72,7 +72,6 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
       return (data_unset *)ds;
     } else {
       fprintf(stderr, "data type mismatch, cannot be merge\n");
-      op1->free(op1);
       return NULL;
     }
   }
@@ -106,7 +105,7 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
 }
 
 
-#line 110 "configparser.c"
+#line 109 "configparser.c"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -515,42 +514,42 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 25:
 #line 142 "./configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 518 "configparser.c"
+#line 517 "configparser.c"
       break;
     case 35:
 #line 133 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 523 "configparser.c"
+#line 522 "configparser.c"
       break;
     case 36:
 #line 134 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 528 "configparser.c"
+#line 527 "configparser.c"
       break;
     case 37:
 #line 135 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 533 "configparser.c"
+#line 532 "configparser.c"
       break;
     case 39:
 #line 136 "./configparser.y"
 { array_free((yypminor->yy40)); }
-#line 538 "configparser.c"
+#line 537 "configparser.c"
       break;
     case 40:
 #line 137 "./configparser.y"
 { array_free((yypminor->yy40)); }
-#line 543 "configparser.c"
+#line 542 "configparser.c"
       break;
     case 41:
 #line 138 "./configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 548 "configparser.c"
+#line 547 "configparser.c"
       break;
     case 42:
 #line 139 "./configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 553 "configparser.c"
+#line 552 "configparser.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -818,9 +817,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 116 "./configparser.y"
+#line 115 "./configparser.y"
 { yymsp[-1].minor.yy78 = NULL; }
-#line 823 "configparser.c"
+#line 822 "configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -835,31 +834,33 @@ static void yy_reduce(
       case 9:
 #line 144 "./configparser.y"
 {
-  buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
-  if (strncmp(yymsp[-2].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
-    fprintf(stderr, "Setting env variable is not supported in conditional %d %s: %s\n",
-        ctx->current->context_ndx,
-        ctx->current->key->ptr, yymsp[-2].minor.yy43->ptr);
-    ctx->ok = 0;
-  } else if (NULL == array_get_element(ctx->current->value, yymsp[0].minor.yy41->key->ptr)) {
-    array_insert_unique(ctx->current->value, yymsp[0].minor.yy41);
-    yymsp[0].minor.yy41 = NULL;
-  } else {
-    fprintf(stderr, "Duplicate config variable in conditional %d %s: %s\n",
-            ctx->current->context_ndx,
-            ctx->current->key->ptr, yymsp[0].minor.yy41->key->ptr);
-    ctx->ok = 0;
-    yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
-    yymsp[0].minor.yy41 = NULL;
+  if (ctx->ok) {
+    buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
+    if (strncmp(yymsp[-2].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
+      fprintf(stderr, "Setting env variable is not supported in conditional %d %s: %s\n",
+          ctx->current->context_ndx,
+          ctx->current->key->ptr, yymsp[-2].minor.yy43->ptr);
+      ctx->ok = 0;
+    } else if (NULL == array_get_element(ctx->current->value, yymsp[0].minor.yy41->key->ptr)) {
+      array_insert_unique(ctx->current->value, yymsp[0].minor.yy41);
+      yymsp[0].minor.yy41 = NULL;
+    } else {
+      fprintf(stderr, "Duplicate config variable in conditional %d %s: %s\n",
+              ctx->current->context_ndx,
+              ctx->current->key->ptr, yymsp[0].minor.yy41->key->ptr);
+      ctx->ok = 0;
+      yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
+      yymsp[0].minor.yy41 = NULL;
+    }
   }
   buffer_free(yymsp[-2].minor.yy43);
   yymsp[-2].minor.yy43 = NULL;
 }
-#line 858 "configparser.c"
+#line 859 "configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 166 "./configparser.y"
+#line 168 "./configparser.y"
 {
   array *vars = ctx->current->value;
   data_unset *du;
@@ -884,6 +885,7 @@ static void yy_reduce(
     du = configparser_merge_data(du, yymsp[0].minor.yy41);
     if (NULL == du) {
       ctx->ok = 0;
+      du->free(du);
     }
     else {
       buffer_copy_string_buffer(du->key, yymsp[-2].minor.yy43);
@@ -898,11 +900,11 @@ static void yy_reduce(
   yymsp[-2].minor.yy43 = NULL;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 901 "configparser.c"
+#line 903 "configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 205 "./configparser.y"
+#line 208 "./configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     yygotominor.yy43 = buffer_init_string("var.");
@@ -914,10 +916,10 @@ static void yy_reduce(
     yymsp[0].minor.yy0 = NULL;
   }
 }
-#line 917 "configparser.c"
+#line 919 "configparser.c"
         break;
       case 12:
-#line 217 "./configparser.y"
+#line 220 "./configparser.y"
 {
   yygotominor.yy41 = configparser_merge_data(yymsp[-2].minor.yy41, yymsp[0].minor.yy41);
   if (NULL == yygotominor.yy41) {
@@ -927,20 +929,21 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 930 "configparser.c"
+#line 932 "configparser.c"
   yy_destructor(5,&yymsp[-1].minor);
         break;
       case 13:
-#line 227 "./configparser.y"
+#line 230 "./configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 939 "configparser.c"
+#line 941 "configparser.c"
         break;
       case 14:
-#line 232 "./configparser.y"
+#line 235 "./configparser.y"
 {
+  yygotominor.yy41 = NULL;
   if (strncmp(yymsp[0].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
     char *env;
 
@@ -965,59 +968,59 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy43);
   yymsp[0].minor.yy43 = NULL;
 }
-#line 968 "configparser.c"
+#line 971 "configparser.c"
         break;
       case 15:
-#line 258 "./configparser.y"
+#line 262 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_string_init();
   buffer_copy_string_buffer(((data_string *)(yygotominor.yy41))->value, yymsp[0].minor.yy0);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 978 "configparser.c"
+#line 981 "configparser.c"
         break;
       case 16:
-#line 265 "./configparser.y"
+#line 269 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_integer_init();
   ((data_integer *)(yygotominor.yy41))->value = strtol(yymsp[0].minor.yy0->ptr, NULL, 10);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 988 "configparser.c"
+#line 991 "configparser.c"
         break;
       case 17:
-#line 271 "./configparser.y"
+#line 275 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_array_init();
   array_free(((data_array *)(yygotominor.yy41))->value);
   ((data_array *)(yygotominor.yy41))->value = yymsp[0].minor.yy40;
   yymsp[0].minor.yy40 = NULL;
 }
-#line 998 "configparser.c"
+#line 1001 "configparser.c"
         break;
       case 18:
-#line 277 "./configparser.y"
+#line 281 "./configparser.y"
 {
   yygotominor.yy40 = array_init();
 }
-#line 1005 "configparser.c"
+#line 1008 "configparser.c"
   yy_destructor(8,&yymsp[-1].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 19:
-#line 280 "./configparser.y"
+#line 284 "./configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1015 "configparser.c"
+#line 1018 "configparser.c"
   yy_destructor(8,&yymsp[-2].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 20:
-#line 285 "./configparser.y"
+#line 289 "./configparser.y"
 {
   if (buffer_is_empty(yymsp[0].minor.yy41->key) ||
       NULL == array_get_element(yymsp[-2].minor.yy40, yymsp[0].minor.yy41->key->ptr)) {
@@ -1034,37 +1037,37 @@ static void yy_reduce(
   yygotominor.yy40 = yymsp[-2].minor.yy40;
   yymsp[-2].minor.yy40 = NULL;
 }
-#line 1037 "configparser.c"
+#line 1040 "configparser.c"
   yy_destructor(10,&yymsp[-1].minor);
         break;
       case 21:
-#line 302 "./configparser.y"
+#line 306 "./configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1046 "configparser.c"
+#line 1049 "configparser.c"
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 22:
-#line 307 "./configparser.y"
+#line 311 "./configparser.y"
 {
   yygotominor.yy40 = array_init();
   array_insert_unique(yygotominor.yy40, yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1056 "configparser.c"
+#line 1059 "configparser.c"
         break;
       case 23:
-#line 313 "./configparser.y"
+#line 317 "./configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1064 "configparser.c"
+#line 1067 "configparser.c"
         break;
       case 24:
-#line 317 "./configparser.y"
+#line 321 "./configparser.y"
 {
   buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
   buffer_free(yymsp[-2].minor.yy43);
@@ -1073,7 +1076,7 @@ static void yy_reduce(
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1076 "configparser.c"
+#line 1079 "configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 25:
@@ -1082,58 +1085,18 @@ static void yy_reduce(
       case 26:
         break;
       case 27:
-#line 329 "./configparser.y"
+#line 333 "./configparser.y"
 {
   data_config *dc;
   dc = (data_config *)array_get_element(ctx->srv->config_context, "global");
   assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1092 "configparser.c"
+#line 1095 "configparser.c"
   yy_destructor(12,&yymsp[0].minor);
         break;
       case 28:
-#line 336 "./configparser.y"
-{
-  data_config *cur;
-
-  cur = ctx->current;
-  configparser_pop(ctx);
-
-  assert(cur && ctx->current);
-
-  yygotominor.yy0 = cur;
-}
-#line 1107 "configparser.c"
-        /* No destructor defined for globalstart */
-  yy_destructor(13,&yymsp[-2].minor);
-        /* No destructor defined for metalines */
-  yy_destructor(14,&yymsp[0].minor);
-        break;
-      case 29:
-#line 347 "./configparser.y"
-{
-  assert(yymsp[-3].minor.yy78->context_ndx < yymsp[0].minor.yy78->context_ndx);
-  yymsp[0].minor.yy78->prev = yymsp[-3].minor.yy78;
-  yymsp[-3].minor.yy78->next = yymsp[0].minor.yy78;
-  yygotominor.yy78 = yymsp[0].minor.yy78;
-  yymsp[-3].minor.yy78 = NULL;
-  yymsp[0].minor.yy78 = NULL;
-}
-#line 1123 "configparser.c"
-        /* No destructor defined for eols */
-  yy_destructor(15,&yymsp[-1].minor);
-        break;
-      case 30:
-#line 356 "./configparser.y"
-{
-  yygotominor.yy78 = yymsp[0].minor.yy78;
-  yymsp[0].minor.yy78 = NULL;
-}
-#line 1133 "configparser.c"
-        break;
-      case 31:
-#line 361 "./configparser.y"
+#line 340 "./configparser.y"
 {
   data_config *cur;
 
@@ -1144,14 +1107,54 @@ static void yy_reduce(
 
   yygotominor.yy78 = cur;
 }
-#line 1147 "configparser.c"
+#line 1110 "configparser.c"
+        /* No destructor defined for globalstart */
+  yy_destructor(13,&yymsp[-2].minor);
+        /* No destructor defined for metalines */
+  yy_destructor(14,&yymsp[0].minor);
+        break;
+      case 29:
+#line 351 "./configparser.y"
+{
+  assert(yymsp[-3].minor.yy78->context_ndx < yymsp[0].minor.yy78->context_ndx);
+  yymsp[0].minor.yy78->prev = yymsp[-3].minor.yy78;
+  yymsp[-3].minor.yy78->next = yymsp[0].minor.yy78;
+  yygotominor.yy78 = yymsp[0].minor.yy78;
+  yymsp[-3].minor.yy78 = NULL;
+  yymsp[0].minor.yy78 = NULL;
+}
+#line 1126 "configparser.c"
+        /* No destructor defined for eols */
+  yy_destructor(15,&yymsp[-1].minor);
+        break;
+      case 30:
+#line 360 "./configparser.y"
+{
+  yygotominor.yy78 = yymsp[0].minor.yy78;
+  yymsp[0].minor.yy78 = NULL;
+}
+#line 1136 "configparser.c"
+        break;
+      case 31:
+#line 365 "./configparser.y"
+{
+  data_config *cur;
+
+  cur = ctx->current;
+  configparser_pop(ctx);
+
+  assert(cur && ctx->current);
+
+  yygotominor.yy78 = cur;
+}
+#line 1150 "configparser.c"
         /* No destructor defined for context */
   yy_destructor(13,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(14,&yymsp[0].minor);
         break;
       case 32:
-#line 372 "./configparser.y"
+#line 376 "./configparser.y"
 {
   data_config *dc;
   buffer *b, *rvalue, *op;
@@ -1200,10 +1203,15 @@ static void yy_reduce(
       { COMP_HTTP_URL,           CONST_STR_LEN("HTTP[\"url\"]"        ) },
       { COMP_HTTP_HOST,          CONST_STR_LEN("HTTP[\"host\"]"       ) },
       { COMP_HTTP_REFERER,       CONST_STR_LEN("HTTP[\"referer\"]"    ) },
-      { COMP_HTTP_USERAGENT,     CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
+      { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
+      { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"user-agent\"]"  ) },
       { COMP_HTTP_COOKIE,        CONST_STR_LEN("HTTP[\"cookie\"]"     ) },
-      { COMP_HTTP_REMOTEIP,      CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
-      { COMP_HTTP_QUERYSTRING,   CONST_STR_LEN("HTTP[\"querystring\"]") },
+      { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
+      { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remote-ip\"]"   ) },
+      { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"querystring\"]") },
+      { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"query-string\"]") },
+      { COMP_HTTP_REQUEST_METHOD, CONST_STR_LEN("HTTP[\"request-method\"]") },
+      { COMP_HTTP_SCHEME,        CONST_STR_LEN("HTTP[\"scheme\"]"     ) },
       { COMP_UNSET, NULL, 0 },
     };
     size_t i;
@@ -1287,45 +1295,45 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1290 "configparser.c"
+#line 1298 "configparser.c"
   yy_destructor(16,&yymsp[-6].minor);
   yy_destructor(18,&yymsp[-4].minor);
   yy_destructor(19,&yymsp[-2].minor);
         break;
       case 33:
-#line 507 "./configparser.y"
+#line 516 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_EQ;
 }
-#line 1300 "configparser.c"
+#line 1308 "configparser.c"
   yy_destructor(20,&yymsp[0].minor);
         break;
       case 34:
-#line 510 "./configparser.y"
+#line 519 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_MATCH;
 }
-#line 1308 "configparser.c"
+#line 1316 "configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 35:
-#line 513 "./configparser.y"
+#line 522 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NE;
 }
-#line 1316 "configparser.c"
+#line 1324 "configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 36:
-#line 516 "./configparser.y"
+#line 525 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NOMATCH;
 }
-#line 1324 "configparser.c"
+#line 1332 "configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 37:
-#line 520 "./configparser.y"
+#line 529 "./configparser.y"
 {
   yygotominor.yy43 = NULL;
   if (ctx->ok) {
@@ -1342,10 +1350,10 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1345 "configparser.c"
+#line 1353 "configparser.c"
         break;
       case 38:
-#line 537 "./configparser.y"
+#line 546 "./configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1355,11 +1363,11 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1358 "configparser.c"
+#line 1366 "configparser.c"
   yy_destructor(24,&yymsp[-1].minor);
         break;
       case 39:
-#line 547 "./configparser.y"
+#line 556 "./configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1369,7 +1377,7 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1372 "configparser.c"
+#line 1380 "configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
   };
@@ -1399,11 +1407,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 107 "./configparser.y"
+#line 106 "./configparser.y"
 
   ctx->ok = 0;
 
-#line 1406 "configparser.c"
+#line 1414 "configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
