@@ -394,10 +394,13 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			 * 403 is from the response handler when noone else catched it
 			 *
 			 * */
-			if (con->uri.path->used &&
+			if ((!con->http_status || con->http_status == 200) && con->uri.path->used &&
 			    con->uri.path->ptr[0] != '*') {
 				response_header_insert(srv, con, CONST_STR_LEN("Allow"), CONST_STR_LEN("OPTIONS, GET, HEAD, POST"));
 
+				con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
+				con->parsed_response &= ~HTTP_CONTENT_LENGTH;
+
 				con->http_status = 200;
 				con->file_finished = 1;
 
@@ -498,9 +501,11 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 	case 207:
 	case 200: /* class: header + body */
 	case 201:
+	case 300:
 	case 301:
 	case 302:
 	case 303:
+	case 307:
 		break;
 
 	case 206: /* write_queue is already prepared */
@@ -511,13 +516,13 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 	default:
 		/* disable chunked encoding again as we have no body */
 		con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
+		con->parsed_response &= ~HTTP_CONTENT_LENGTH;
 		chunkqueue_reset(con->write_queue);
 
 		con->file_finished = 1;
 		break;
 	}
 
-
 	if (con->file_finished) {
 		/* we have all the content and chunked encoding is not used, set a content-length */
 
@@ -536,12 +541,16 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 			if ((con->http_status >= 100 && con->http_status < 200) ||
 			    con->http_status == 204 ||
 			    con->http_status == 304) {
+				data_string *ds;
 				/* no Content-Body, no Content-Length */
-			} else if (qlen >= 0) {
+				if (NULL != (ds = (data_string*) array_get_element(con->response.headers, "Content-Length"))) {
+					buffer_reset(ds->value); // Headers with empty values are ignored for output
+				}
+			} else if (qlen > 0 || con->request.http_method != HTTP_METHOD_HEAD) {
 				/* qlen = 0 is important for Redirects (301, ...) as they MAY have
 				 * a content. Browsers are waiting for a Content otherwise
 				 */
-				buffer_copy_off_t(srv->tmp_buf, chunkqueue_length(con->write_queue));
+				buffer_copy_off_t(srv->tmp_buf, qlen);
 
 				response_header_overwrite(srv, con, CONST_STR_LEN("Content-Length"), CONST_BUF_LEN(srv->tmp_buf));
 			}
@@ -582,6 +591,8 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		 * a HEAD request has the same as a GET 
 		 * without the content
 		 */
+		con->file_finished = 1;
+
 		chunkqueue_reset(con->write_queue);
 		con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
 	}
@@ -834,14 +845,8 @@ int connection_reset(server *srv, connection *con) {
 		con->plugin_ctx[pd->id] = NULL;
 	}
 
-#if COND_RESULT_UNSET
-	for (i = srv->config_context->used - 1; i >= 0; i --) {
-		con->cond_cache[i].result = COND_RESULT_UNSET;
-		con->cond_cache[i].patterncount = 0;
-	}
-#else
-	memset(con->cond_cache, 0, sizeof(cond_cache_t) * srv->config_context->used);
-#endif
+	/* The cond_cache gets reset in response.c */
+//	config_cond_cache_reset(srv, con);
 
 #ifdef USE_OPENSSL
 	if (con->ssl_error_want_reuse_buffer) {
