@@ -29,6 +29,7 @@
 #include "log.h"
 #include "http_auth.h"
 #include "http_auth_digest.h"
+#include "inet_ntop_cache.h"
 #include "stream.h"
 
 #ifdef USE_OPENSSL
@@ -831,19 +832,17 @@ int http_auth_basic_check(server *srv, connection *con, mod_auth_plugin_data *p,
 	username = buffer_init();
 
 	if (!base64_decode(username, realm_str)) {
-		buffer_free(username);
-
 		log_error_write(srv, __FILE__, __LINE__, "sb", "decodeing base64-string failed", username);
 
+		buffer_free(username);
 		return 0;
 	}
 
 	/* r2 == user:password */
 	if (NULL == (pw = strchr(username->ptr, ':'))) {
-		buffer_free(username);
-
 		log_error_write(srv, __FILE__, __LINE__, "sb", ": is missing in", username);
 
+		buffer_free(username);
 		return 0;
 	}
 
@@ -864,7 +863,7 @@ int http_auth_basic_check(server *srv, connection *con, mod_auth_plugin_data *p,
 
 	/* password doesn't match */
 	if (http_auth_basic_password_compare(srv, p, req, username, realm->value, password, pw)) {
-		log_error_write(srv, __FILE__, __LINE__, "sbb", "password doesn't match for", con->uri.path, username);
+		log_error_write(srv, __FILE__, __LINE__, "sbbss", "password doesn't match for ", con->uri.path, username, ", IP:", inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
 
 		buffer_free(username);
 		buffer_free(password);
@@ -1132,7 +1131,7 @@ int http_auth_digest_check(server *srv, connection *con, mod_auth_plugin_data *p
 		}
 
 		log_error_write(srv, __FILE__, __LINE__, "sss",
-				"digest: auth failed for", username, "wrong password");
+				"digest: auth failed for ", username, ": wrong password, IP:", inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
 
 		buffer_free(b);
 		return 0;
