@@ -227,9 +227,9 @@ URIHANDLER_FUNC(mod_usertrack_uri_handler) {
 	MD5_Update(&Md5Ctx, (unsigned char *)"+", 1);
 
 	/* we assume sizeof(time_t) == 4 here, but if not it ain't a problem at all */
-	ltostr(hh, srv->cur_ts);
+	LI_ltostr(hh, srv->cur_ts);
 	MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
-	ltostr(hh, rand());
+	LI_ltostr(hh, rand());
 	MD5_Update(&Md5Ctx, (unsigned char *)hh, strlen(hh));
 
 	MD5_Final(h, &Md5Ctx);
