@@ -922,6 +922,9 @@ int http_request_parse(server *srv, connection *con) {
 								} else if (0 == strcasecmp(con->request.http_if_modified_since,
 											ds->value->ptr)) {
 									/* ignore it if they are the same */
+
+									ds->free((data_unset *)ds);
+									ds = NULL;
 								} else {
 									con->http_status = 400;
 									con->keep_alive = 0;
@@ -977,7 +980,7 @@ int http_request_parse(server *srv, connection *con) {
 								}
 							}
 
-							array_insert_unique(con->request.headers, (data_unset *)ds);
+							if (ds) array_insert_unique(con->request.headers, (data_unset *)ds);
 						} else {
 							/* empty header-fields are not allowed by HTTP-RFC, we just ignore them */
 						}
@@ -1011,6 +1014,17 @@ int http_request_parse(server *srv, connection *con) {
 				/* strip leading WS */
 				if (value == cur) value = cur+1;
 			default:
+				if (*cur >= 0 && *cur < 32) {
+					if (srv->srvconf.log_request_header_on_error) {
+						log_error_write(srv, __FILE__, __LINE__, "sds",
+								"invalid char in header", (int)*cur, "-> 400");
+					}
+
+					con->http_status = 400;
+					con->keep_alive = 0;
+
+					return 0;
+				}
 				break;
 			}
 		}
