@@ -2,8 +2,8 @@
 
 use strict;
 
-use Test::Harness qw(&runtests $verbose); 
-$verbose=0; 
+use Test::Harness qw(&runtests $verbose);
+$verbose = (defined $ENV{'VERBOSE'} ? $ENV{'VERBOSE'} : 0);;
 
 my $srcdir = (defined $ENV{'srcdir'} ? $ENV{'srcdir'} : '.');
 
